@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.5.1 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5.3 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -36,7 +36,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: check.c,v 1.116 1996/09/08 00:21:42 millert Exp $";
+static char rcsid[] = "$Id: check.c,v 1.122 1996/11/14 02:37:16 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -89,6 +89,9 @@ static char rcsid[] = "$Id: check.c,v 1.116 1996/09/08 00:21:42 millert Exp $";
 #ifdef HAVE_SKEY
 #  include <skey.h>
 #endif /* HAVE_SKEY */
+#ifdef HAVE_OPIE
+#  include <opie.h>
+#endif /* HAVE_OPIE */
 #ifdef HAVE_UTIME
 #  ifdef HAVE_UTIME_H
 #    include <utime.h>
@@ -112,6 +115,9 @@ static int   sudo_krb_validate_user	__P((struct passwd *, char *));
 #ifdef HAVE_SKEY
 static char *sudo_skeyprompt		__P((struct skey *, char *));
 #endif /* HAVE_SKEY */
+#ifdef HAVE_OPIE
+static char *sudo_opieprompt		__P((struct opie *, char *));
+#endif /* HAVE_OPIE */
 int   user_is_exempt			__P((void));
 
 /*
@@ -125,6 +131,9 @@ union config_record configure;
 #ifdef HAVE_SKEY
 struct skey skey;
 #endif
+#ifdef HAVE_OPIE
+struct opie opie;
+#endif
 #if (SHADOW_TYPE == SPW_SECUREWARE) && defined(__alpha)
 extern uchar_t crypt_type;
 #endif /* SPW_SECUREWARE && __alpha */
@@ -218,15 +227,27 @@ static int check_timestamp()
     else
 	p = tty;
 
+    if (sizeof(_PATH_SUDO_TIMEDIR) + strlen(user_name) + strlen(p) + 2 >
+	sizeof(timestampfile)) {
+	(void) fprintf(stderr, "%s:  path too long:  %s/%s.%s\n", Argv[0],
+		       _PATH_SUDO_TIMEDIR, user_name, p);
+	exit(1);                                              
+    }
     (void) sprintf(timestampfile, "%s/%s.%s", _PATH_SUDO_TIMEDIR, user_name, p);
 #else
+    if (sizeof(_PATH_SUDO_TIMEDIR) + strlen(user_name) + 1 >
+	sizeof(timestampfile)) {
+	(void) fprintf(stderr, "%s:  path too long:  %s/%s\n", Argv[0],
+		       _PATH_SUDO_TIMEDIR, user_name);
+	exit(1);                                              
+    }
     (void) sprintf(timestampfile, "%s/%s", _PATH_SUDO_TIMEDIR, user_name);
 #endif /* USE_TTY_TICKETS */
 
     timedir_is_good = 1;	/* now there's an assumption for ya... */
 
     /* become root */
-    set_perms(PERM_ROOT);
+    set_perms(PERM_ROOT, 0);
 
     /*
      * walk through the path one directory at a time
@@ -289,7 +310,7 @@ static int check_timestamp()
     }
 
     /* relinquish root */
-    set_perms(PERM_USER);
+    set_perms(PERM_USER, 0);
 
     return (timestamp_is_old);
 }
@@ -341,7 +362,7 @@ static void update_timestamp()
 {
     if (timedir_is_good) {
 	/* become root */
-	set_perms(PERM_ROOT);
+	set_perms(PERM_ROOT, 0);
 
 	if (touch(timestampfile) < 0) {
 	    int fd = open(timestampfile, O_WRONLY | O_CREAT | O_TRUNC, 0600);
@@ -353,7 +374,7 @@ static void update_timestamp()
 	}
 
 	/* relinquish root */
-	set_perms(PERM_USER);
+	set_perms(PERM_USER, 0);
     }
 }
 
@@ -376,19 +397,31 @@ void remove_timestamp()
     else
 	p = tty;
 
+    if (sizeof(_PATH_SUDO_TIMEDIR) + strlen(user_name) + strlen(p) + 2 >
+	sizeof(timestampfile)) {
+	(void) fprintf(stderr, "%s:  path too long:  %s/%s.%s\n", Argv[0],
+		       _PATH_SUDO_TIMEDIR, user_name, p);
+	exit(1);                                              
+    }
     (void) sprintf(timestampfile, "%s/%s.%s", _PATH_SUDO_TIMEDIR, user_name, p);
 #else
+    if (sizeof(_PATH_SUDO_TIMEDIR) + strlen(user_name) + 1 >
+	sizeof(timestampfile)) {
+	(void) fprintf(stderr, "%s:  path too long:  %s/%s\n", Argv[0],
+		       _PATH_SUDO_TIMEDIR, user_name);
+	exit(1);                                              
+    }
     (void) sprintf(timestampfile, "%s/%s", _PATH_SUDO_TIMEDIR, user_name);
 #endif /* USE_TTY_TICKETS */
 
     /* become root */
-    set_perms(PERM_ROOT);
+    set_perms(PERM_ROOT, 0);
 
     /* remove the ticket file */
     (void) unlink(timestampfile);
 
     /* relinquish root */
-    set_perms(PERM_USER);
+    set_perms(PERM_USER, 0);
 }
 
 
@@ -411,7 +444,7 @@ static void check_passwd()
     sd = &sd_dat;
 
     /* Initialize SecurID. */
-    set_perms(PERM_ROOT);
+    set_perms(PERM_ROOT, 0);
     creadcfg();
     if (sd_init(sd) != 0) {
 	(void) fprintf(stderr, "%s: Cannot contact SecurID server\n", Argv[0]);
@@ -423,7 +456,7 @@ static void check_passwd()
      */
     while (counter > 0) {
 	if (sd_auth(sd) == ACM_OK) {
-	    set_perms(PERM_USER);
+	    set_perms(PERM_USER, 0);
 	    return;
 	}
 
@@ -434,7 +467,7 @@ static void check_passwd()
 	(void) fprintf(stderr, "%s\n", INCORRECT_PASSWORD);
 #endif /* USE_INSULTS */
     }
-    set_perms(PERM_USER);
+    set_perms(PERM_USER, 0);
 
     if (counter > 0) {
 	log_error(PASSWORD_NOT_CORRECT);
@@ -458,6 +491,9 @@ static void check_passwd()
 #ifdef HAVE_SKEY
     (void) memset((VOID *)&skey, 0, sizeof(skey));
 #endif /* HAVE_SKEY */
+#ifdef HAVE_OPIE
+    (void) memset((VOID *)&opie, 0, sizeof(opie));
+#endif /* HAVE_OPIE */
 
     /*
      * you get TRIES_FOR_PASSWORD times to guess your password
@@ -466,10 +502,16 @@ static void check_passwd()
 
 #ifdef HAVE_SKEY
 	/* rewrite the prompt if using s/key since the challenge can change */
-	set_perms(PERM_ROOT);
+	set_perms(PERM_ROOT, 0);
 	prompt = sudo_skeyprompt(&skey, prompt);
-	set_perms(PERM_USER);
+	set_perms(PERM_USER, 0);
 #endif /* HAVE_SKEY */
+#ifdef HAVE_OPIE
+	/* rewrite the prompt if using OPIE since the challenge can change */
+	set_perms(PERM_ROOT, 0);
+	prompt = sudo_opieprompt(&opie, prompt);
+	set_perms(PERM_USER, 0);
+#endif /* HAVE_OPIE */
 
 	/* get a password from the user */
 #ifdef USE_GETPASS
@@ -494,14 +536,25 @@ static void check_passwd()
 #ifdef HAVE_SKEY
 	/* Only check s/key db if the user exists there */
 	if (skey.keyfile) {
-	    set_perms(PERM_ROOT);
+	    set_perms(PERM_ROOT, 0);
 	    if (skeyverify(&skey, pass) == 0) {
-		set_perms(PERM_USER);
+		set_perms(PERM_USER, 0);
 		return;             /* if the key is correct return() */
 	    }
-	    set_perms(PERM_USER);
+	    set_perms(PERM_USER, 0);
 	}
 #endif /* HAVE_SKEY */
+#ifdef HAVE_OPIE
+	/* Only check OPIE db if the user exists there */
+	if (opie.opie_flags) {
+	    set_perms(PERM_ROOT, 0);
+	    if (opieverify(&opie, pass) == 0) {
+		set_perms(PERM_USER, 0);
+		return;             /* if the key is correct return() */
+	    }
+	    set_perms(PERM_USER, 0);
+	}
+#endif /* HAVE_OPIE */
 #if !defined(HAVE_SKEY) || !defined(SKEY_ONLY)
 	/*
 	 * If we use shadow passwords with a different crypt(3)
@@ -625,7 +678,7 @@ static int sudo_krb_validate_user(pw_ent, pass)
      * Update the ticket if password is ok.  Kerb4 expects
      * the ruid and euid to be the same here so we setuid to root.
      */
-    set_perms(PERM_ROOT);
+    set_perms(PERM_ROOT, 0);
     k_errno = krb_get_pw_in_tkt(pw_ent->pw_name, "", realm, "krbtgt", realm,
 	DEFAULT_TKT_LIFE, pass);
 
@@ -640,7 +693,7 @@ static int sudo_krb_validate_user(pw_ent, pass)
 		       krb_err_txt[k_errno]);
 
     /* done with rootly stuff */
-    set_perms(PERM_USER);
+    set_perms(PERM_USER, 0);
 
     return(!(k_errno == INTK_OK));
 }
@@ -660,79 +713,147 @@ static char *sudo_skeyprompt(user_skey, p)
     struct skey *user_skey;
     char *p;
 {
-#ifndef LONG_SKEY_PROMPT
-    char buf[32];
-    char *new_prompt;
-    static char *old_prompt = NULL;
-    static int plen;
-#endif /* LONG_SKEY_PROMPT */
+    char challenge[256];
+    int rval;
+    static char *orig_prompt = NULL, *new_prompt = NULL;
+    static int op_len, np_size;
 
-    /* close the key file if necesary */
-    if (user_skey->keyfile != NULL)
+    /* save the original prompt */
+    if (orig_prompt == NULL) {
+	orig_prompt = p;
+	op_len = strlen(p);
+
+	/* ignore trailing colon */
+	if (p[op_len - 1] == ':')
+	    op_len--;
+    }
+
+    /* close old stream */
+    if (user_skey->keyfile)
 	(void) fclose(user_skey->keyfile);
 
-    /* return the old prompt if we cannot get s/key info */
-    if (skeylookup(user_skey, user_name)) {
-	if (user_skey->keyfile != NULL) {
-	    (void) fclose(user_skey->keyfile);
-	    user_skey->keyfile = NULL;
-	}
-#ifdef SKEY_ONLY
-	(void) fprintf(stderr, "%s: You do not exist in the s/key database.\n",
+    /* get the skey part of the prompt */
+    if ((rval = skeychallenge(user_skey, user_name, challenge)) != 0) {
+#ifdef OTP_ONLY
+	(void) fprintf(stderr,
+		       "%s: You do not exist in the s/key database.\n",
 		       Argv[0]);
 	exit(1);
 #else
-#  ifdef LONG_SKEY_PROMPT
-	return(p);
-#  else
-	if (old_prompt == NULL) {
-	    return(p);
-	} else {
-	    return(old_prompt);
+	/* return the original prompt if we cannot get s/key info */
+	return(orig_prompt);
+#endif /* OTP_ONLY */
+    }
+
+    /* get space for new prompt with embedded s/key challenge */
+    if (new_prompt == NULL) {
+	/* allocate space for new prompt */
+	np_size = op_len + strlen(challenge) + 7;
+	if (!(new_prompt = (char *) malloc(np_size))) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
 	}
-#  endif /* LONG_SKEY_PROMPT */
-#endif /* SKEY_ONLY */
-    }
-
-#ifdef LONG_SKEY_PROMPT
-    /* separate s/key challenge and prompt for easy snarfing */
-    (void) fprintf(stderr, "key %d %s\n", user_skey->n - 1, user_skey->seed);
-
-    /* return old prompt unmolested */
-    return(p);
-
-#else
-
-    /* keep a pointer to the original prompt around for future reference */
-    if (old_prompt == NULL) {
-	old_prompt = p;
-	plen = strlen(p);
-
-	/* ignore trailing colon's */
-	if (p[plen - 1] == ':')
-	    plen--;
     } else {
-	(void) free(p);
-    }
-
-    (void) sprintf(buf, "%d", user_skey->n - 1);
-    if ((new_prompt = (char *)
-	malloc(plen + strlen(buf) + strlen(user_skey->seed) + 12)) == NULL) {
-	perror("malloc");
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
+	/* already have space allocated, is it enough? */
+	if (np_size < op_len + strlen(challenge) + 7) {
+	    np_size = op_len + strlen(challenge) + 7;
+	    if (!(new_prompt = (char *) realloc(new_prompt, np_size))) {
+		perror("malloc");
+		(void) fprintf(stderr, "%s: cannot allocate memory!\n",
+			       Argv[0]);
+		exit(1);
+	    }
+	}
     }
 
     /* embed the s/key challenge into the new password prompt */
-    (void) sprintf(new_prompt, "%.*s [s/key %d %s]:", plen, old_prompt,
-	user_skey->n - 1, user_skey->seed);
+#ifdef LONG_OTP_PROMPT
+    (void) sprintf(new_prompt, "%s\n%s", challenge, orig_prompt);
+#else
+    (void) sprintf(new_prompt, "%.*s [ %s ]:", op_len, orig_prompt, challenge);
+#endif /* LONG_OTP_PROMPT */
 
     return(new_prompt);
-#endif /* LONG_SKEY_PROMPT */
 }
 #endif /* HAVE_SKEY */
 
 
+#ifdef HAVE_OPIE
+/********************************************************************
+ *
+ *  sudo_opieprompt()
+ *
+ *  This function rewrites and return the prompt based the
+ *  OPIE challenge *  and fills in the user's opie structure.
+ */
+
+static char *sudo_opieprompt(user_opie, p)
+    struct opie *user_opie;
+    char *p;
+{
+    char challenge[OPIE_CHALLENGE_MAX];
+    int rval;
+    static char *orig_prompt = NULL, *new_prompt = NULL;
+    static int op_len, np_size;
+
+    /* save the original prompt */
+    if (orig_prompt == NULL) {
+	orig_prompt = p;
+	op_len = strlen(p);
+
+	/* ignore trailing colon */
+	if (p[op_len - 1] == ':')
+	    op_len--;
+    }
+
+    /* get the opie part of the prompt */
+    if ((rval = opiechallenge(user_opie, user_name, challenge)) != 0) {
+#ifdef OTP_ONLY
+	(void) fprintf(stderr,
+		       "%s: You do not exist in the s/key database.\n",
+		       Argv[0]);
+	exit(1);
+#else
+	/* return the original prompt if we cannot get s/key info */
+	return(orig_prompt);
+#endif /* OTP_ONLY */
+    }
+
+    /* get space for new prompt with embedded s/key challenge */
+    if (new_prompt == NULL) {
+	/* allocate space for new prompt */
+	np_size = op_len + strlen(challenge) + 7;
+	if (!(new_prompt = (char *) malloc(np_size))) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
+	}
+    } else {
+	/* already have space allocated, is it enough? */
+	if (np_size < op_len + strlen(challenge) + 7) {
+	    np_size = op_len + strlen(challenge) + 7;
+	    if (!(new_prompt = (char *) realloc(new_prompt, np_size))) {
+		perror("malloc");
+		(void) fprintf(stderr, "%s: cannot allocate memory!\n",
+			       Argv[0]);
+		exit(1);
+	    }
+	}
+    }
+
+    /* embed the s/key challenge into the new password prompt */
+#ifdef LONG_OTP_PROMPT
+    (void) sprintf(new_prompt, "%s\n%s", challenge, orig_prompt);
+#else
+    (void) sprintf(new_prompt, "%.*s [ %s ]:", op_len, orig_prompt, challenge);
+#endif /* LONG_OTP_PROMPT */
+
+    return(new_prompt);
+}
+#endif /* HAVE_OPIE */
+
+
 #ifndef NO_MESSAGE
 /********************************************************************
  *
@@ -747,7 +868,7 @@ static void reminder()
     (void) fprintf(stderr, "\n%s\n%s\n\n%s\n%s\n\n",
 #else
     (void) fprintf(stderr, "\n%s\n%s\n%s\n%s\n\n%s\n%s\n\n%s\n%s\n\n",
-	"    CU sudo version 1.5.1, based on Root Group sudo version 1.1",
+	"    CU sudo version 1.5.3, based on Root Group sudo version 1.1",
 	"    sudo version 1.1, Copyright (C) 1991 The Root Group, Inc.",
 	"    sudo comes with ABSOLUTELY NO WARRANTY.  This is free software,",
 	"    and you are welcome to redistribute it under certain conditions.",
