@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.1
+ *  CU sudo version 1.5.3
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
  *
- *  $Id: compat.h,v 1.22 1996/09/08 00:21:42 millert Exp $
+ *  $Id: compat.h,v 1.28 1996/11/14 02:37:16 millert Exp $
  */
 
 #ifndef _SUDO_COMPAT_H
@@ -89,7 +89,7 @@
  * For alternate password schemes we need longer passwords.
  * This is a bit, ummm, gross but necesary.
  */
-#if defined(HAVE_KERB4) || defined(HAVE_AFS) || defined(HAVE_DCE) || defined(HAVE_SKEY)
+#if defined(HAVE_KERB4) || defined(HAVE_AFS) || defined(HAVE_DCE) || defined(HAVE_SKEY) || defined(HAVE_OPIE)
 #  undef _PASSWD_LEN
 #  define _PASSWD_LEN		256
 #else
@@ -108,7 +108,7 @@
 #        endif /* SHADOW_TYPE != SPW_NONE */
 #      endif /* PASS_MAX */
 #    endif /* !_PASSWD_LEN */
-#  endif /* HAVE_KERB4 || HAVE_DCE || HAVE_SKEY */
+#  endif /* HAVE_KERB4 || HAVE_AFS || HAVE_DCE || HAVE_SKEY || HAVE_OPIE */
 #endif /* SPW_SECUREWARE */
 
 /*
@@ -122,20 +122,24 @@
 #endif /* GID_NO_CHANGE */
 
 /*
- * Emulate seteuid() and setegid() for HP-UX
- */
-#ifdef __hpux
-#  define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID, UID_NO_CHANGE))
-#  define setegid(_EGID)	(setresgid(GID_NO_CHANGE, _EGID, GID_NO_CHANGE))
-#endif	/* __hpux */
-
-/*
- * Emulate seteuid() and setegid() for AIX
+ * Emulate seteuid() for AIX via setuidx() -- needed for some versions of AIX
  */
 #ifdef _AIX
 #  include <sys/id.h>
 #  define seteuid(_EUID)	(setuidx(ID_EFFECTIVE|ID_REAL, _EUID))
-#  define setegid(_EGID)	(setgidx(ID_EFFECTIVE|ID_REAL, _EGID))
-#endif	/* _AIX */
+#  undef HAVE_SETEUID
+#  define HAVE_SETEUID		1
+#endif /* _AIX */
+
+/*
+ * Emulate seteuid() for HP-UX via setresuid(2) and seteuid(2) for others.
+ */
+#ifndef HAVE_SETEUID
+#  ifdef __hpux
+#    define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID, UID_NO_CHANGE))
+#  else
+#    define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID))
+#  endif /* __hpux */
+#endif /* HAVE_SETEUID */
 
 #endif /* _SUDO_COMPAT_H */
