@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.1
+ *  CU sudo version 1.5.3
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: goodpath.c,v 1.12 1996/09/08 00:21:42 millert Exp $";
+static char rcsid[] = "$Id: goodpath.c,v 1.14 1996/11/14 02:37:16 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -77,12 +77,12 @@ char * sudo_goodpath(path)
 	return(NULL);
 
     /* we need to be root for the stat */
-    set_perms(PERM_ROOT);
+    set_perms(PERM_ROOT, 0);
 
     err = stat(path, &statbuf);
 
     /* discard root perms */
-    set_perms(PERM_USER);
+    set_perms(PERM_USER, 0);
 
     /* stat(3) failed */
     if (err)
