@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.5.1 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5.3 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -25,7 +25,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  $Id: sudo.h,v 1.111 1996/09/08 00:21:42 millert Exp $
+ *  $Id: sudo.h,v 1.114 1996/11/14 20:08:32 millert Exp $
  */
 
 #ifndef _SUDO_SUDO_H
@@ -92,7 +92,7 @@ struct command_match {
 /*
  * Structure containing Cmnd_Alias's if "sudo -l" is used.
  */
-struct command_alias {
+struct generic_alias {
     char *alias;
     char *entries;
     size_t entries_size;
@@ -159,6 +159,7 @@ struct command_alias {
 #define MODE_LIST                00040
 #define MODE_BACKGROUND          00100
 #define MODE_SHELL               00200
+#define MODE_RESET_HOME          00400
 
 /*
  * Used with set_perms()
@@ -201,7 +202,7 @@ void log_error		__P((int));
 void inform_user	__P((int));
 void check_user		__P((void));
 int validate		__P((int));
-void set_perms		__P((int));
+void set_perms		__P((int, int));
 void remove_timestamp	__P((void));
 void load_interfaces	__P((void));
 int yyparse		__P((void));
